from sqlalchemy.sql.schema import ForeignKey
from sqlalchemy import Column, Integer, String, Numeric, DateTime, Boolean, BigInteger
from sqlalchemy.orm import relationship
from kassa.databases import Base


class Payment(Base):
    __tablename__ = 'payments'

    id = Column('id', Integer, primary_key=True, autoincrement=True)
    external_id = Column('external_id', String(length=128))
    type = Column('type', Integer)
    sum = Column('sum', Numeric(12, 2))


class CorrectionInfoTable(Base):
    __tablename__ = 'correction_info'

    external_id = Column('external_id', String(length=128), primary_key=True)
    type = Column('type', String(length=10))
    base_date = Column('base_date', String(length=128))
    base_number = Column('base_number', String(length=128))


class Error(Base):
    __tablename__ = 'errors'

    external_id = Column('external_id', String(length=128), primary_key=True)
    error_id = Column('error_id', String(length=128))
    code = Column('code', Integer)
    text = Column('text', String(length=250))
    type = Column('type', String(length=10))


class Doc(Base):
    __tablename__ = 'docs'
    uuid = Column('uuid', String(length=128), primary_key=True)
    timestamp = Column('timestamp', String(length=128))
    group_code = Column('group_code', String(length=128))
    daemon_code = Column('daemon_code', String(length=128))
    device_code = Column('device_code', String(length=128))
    external_id = Column('external_id', String(length=128))
    callback_url = Column('callback_url', String(length=128))
    status = Column('status', String(length=128))


class Atol(Base):
    __tablename__ = 'atol_receipts'
    uuid = Column('uuid', String(length=128), primary_key=True)
    timestamp = Column('timestamp', String(length=128))
    external_id = Column('external_id', String(length=128))
    status = Column('status', String(length=128))


class Payload(Base):
    __tablename__ = 'payloads'

    external_id = Column('external_id',  String(128), primary_key=True)
    fiscal_receipt_number = Column('fiscal_receipt_number', BigInteger)
    shift_number = Column('shift_number', BigInteger)
    receipt_datetime = Column('receipt_datetime', String(length=128))
    total = Column('total',  Numeric(12, 2))
    fn_number = Column('fn_number', String(length=128))
    ecr_registration_number = Column(
        'ecr_registration_number', String(length=128))
    fiscal_document_number = Column('fiscal_document_number', BigInteger)
    fiscal_document_attribute = Column('fiscal_document_attribute', BigInteger)
    fns_site = Column('fns_site', String(length=128))


"""
class Sell(Base):
    __tablename__ = 'sells'

    id = Column(Integer, primary_key=True, autoincrement=True)
    external_id = Column('external_id',String(128))
    is_refund = Column('is_refund', Boolean)
    service = Column('service', String(length=16))
    timestamp = Column('timestamp', DateTime)
"""
